@@ -1,5 +1,6 @@
 /* kwset.c - search for any of a set of keywords.
-   Copyright 1989, 1998, 2000, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1989, 1998, 2000, 2005, 2007, 2009
+   Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,7 +38,7 @@
 #include "obstack.h"
 
 #ifdef GREP
-extern char *xmalloc();
+# include "xalloc.h"
 # undef malloc
 # define malloc xmalloc
 #endif
